@@ -99,6 +99,7 @@
 #define CLASS_CANNOT_COPY   0x10000000  /* It cannot be copied. */
 #define CLASS_CANNOT_ASSIGN 0x20000000  /* It cannot be assigned. */
 #define CLASS_ALLOW_NONE    0x40000000  /* The class will handle None. */
+#define CLASS_HAS_NONLAZY   0x80000000  /* The class has non-lazy methods. */
 
 #define hasSigSlots(cd)     ((cd)->classflags & CLASS_HAS_SIGSLOTS)
 #define setHasSigSlots(cd)  ((cd)->classflags |= CLASS_HAS_SIGSLOTS)
@@ -149,6 +150,8 @@
 #define setCannotAssign(cd) ((cd)->classflags |= CLASS_CANNOT_ASSIGN)
 #define classHandlesNone(cd)    ((cd)->classflags & CLASS_ALLOW_NONE)
 #define setClassHandlesNone(cd) ((cd)->classflags |= CLASS_ALLOW_NONE)
+#define hasNonlazyMethod(cd)    ((cd)->classflags & CLASS_HAS_NONLAZY)
+#define setHasNonlazyMethod(cd) ((cd)->classflags |= CLASS_HAS_NONLAZY)
 
 #define isPublicDtor(cd)    ((cd)->classflags & SECT_IS_PUBLIC)
 #define setIsPublicDtor(cd) ((cd)->classflags |= SECT_IS_PUBLIC)
@@ -265,8 +268,10 @@
 #define setIsPrivate(o)     ((o)->overflags |= SECT_IS_PRIVATE)
 #define isSlot(o)           ((o)->overflags & SECT_IS_SLOT)
 #define setIsSlot(o)        ((o)->overflags |= SECT_IS_SLOT)
+#define resetIsSlot(o)      ((o)->overflags &= ~SECT_IS_SLOT)
 #define isSignal(o)         ((o)->overflags & SECT_IS_SIGNAL)
 #define setIsSignal(o)      ((o)->overflags |= SECT_IS_SIGNAL)
+#define resetIsSignal(o)    ((o)->overflags &= ~SECT_IS_SIGNAL)
 
 #define isVirtual(o)        ((o)->overflags & OVER_IS_VIRTUAL)
 #define setIsVirtual(o)     ((o)->overflags |= OVER_IS_VIRTUAL)
@@ -734,6 +739,7 @@ typedef struct _moduleDef {
     codeBlock *initcode;                /* Initialisation code. */
     codeBlock *postinitcode;            /* Post-initialisation code. */
     codeBlock *unitcode;                /* Compilation unit code. */
+    codeBlock *unitpostinccode;         /* Compilation unit post-include code. */
     int parts;                          /* The number of parts generated. */
     char *file;                         /* The filename. */
     qualDef *qualifiers;                /* The list of qualifiers. */
@@ -798,6 +804,7 @@ typedef struct _mappedTypeDef {
     ifaceFileDef *iff;                  /* The interface file. */
     struct _memberDef *members;         /* The static member functions. */
     struct _overDef *overs;             /* The static overloads. */
+    codeBlock *typecode;                /* Type code. */
     codeBlock *convfromcode;            /* Convert from C++ code. */
     codeBlock *convtocode;              /* Convert to C++ code. */
     struct _mappedTypeDef *next;        /* Next in the list. */
@@ -985,7 +992,7 @@ typedef struct _mroDef {
 /* A class definition. */
 
 typedef struct _classDef {
-    int classflags;                     /* The class flags. */
+    unsigned classflags;                /* The class flags. */
     int pyqt4_flags;                    /* The PyQt4 specific flags. */
     nameDef *pyname;                    /* The Python name. */
     ifaceFileDef *iff;                  /* The interface file. */
@@ -1121,9 +1128,10 @@ void prOverloadName(FILE *fp, overDef *od);
 void prOverloadDecl(FILE *fp, ifaceFileDef *scope, overDef *od, int defval);
 void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname);
 int prPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd, int sec,
-        int names, int defaults, int in_str);
+        int names, int defaults, int in_str, int is_signal);
 void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad);
 int isIntReturnSlot(memberDef *md);
+int isSSizeReturnSlot(memberDef *md);
 int isLongReturnSlot(memberDef *md);
 int isVoidReturnSlot(memberDef *md);
 int isNumberSlot(memberDef *md);
@@ -1156,6 +1164,7 @@ typedef enum {
     opt_name_flag,
     dotted_name_flag,
     integer_flag,
+    opt_integer_flag,
     api_range_flag
 } flagType;
 
