@@ -1,18 +1,17 @@
 /*
  * The SIP module interface.
  *
- * Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
- * 
+ * Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+ *
  * This file is part of SIP.
- * 
+ *
  * This copy of SIP is licensed for use under the terms of the SIP License
  * Agreement.  See the file LICENSE for more details.
- * 
+ *
  * This copy of SIP may also used under the terms of the GNU General Public
  * License v2 or v3 as published by the Free Software Foundation which can be
- * found in the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this
- * package.
- * 
+ * found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+ *
  * SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
@@ -55,8 +54,8 @@ extern "C" {
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x040901
-#define SIP_VERSION_STR     "4.9.1"
+#define SIP_VERSION         0x040b02
+#define SIP_VERSION_STR     "4.11.2"
 
 
 /*
@@ -69,6 +68,23 @@ extern "C" {
  *
  * History:
  *
+ * 8.0  Changed the size of the sipSimpleWrapper structure.
+ *      Added sip_api_get_address().
+ *
+ * 7.1  Added the 'H' format character to sip_api_parse_result().
+ *      Deprecated the 'D' format character of sip_api_parse_result().
+ *
+ * 7.0  Added sip_api_parse_kwd_args().
+ *      Added sipErrorState, sip_api_add_exception().
+ *      The type initialisation function is now passed a dictionary of keyword
+ *      arguments.
+ *      All argument parsers now update a set of error messages rather than an
+ *      argument count.
+ *      The signatures of sip_api_no_function() and sip_api_no_method() have
+ *      changed.
+ *      Added ctd_docstring to sipClassTypeDef.
+ *      Added vf_docstring to sipVersionedFunctionDef.
+ *
  * 6.0  Added the sipContainerDef structure to define the contents of a class
  *      or mapped type.  Restructured sipClassDef and sipMappedTypeDef
  *      accordingly.
@@ -155,10 +171,20 @@ extern "C" {
  *
  * 0.0  Original version.
  */
-#define SIP_API_MAJOR_NR    6
+#define SIP_API_MAJOR_NR    8
 #define SIP_API_MINOR_NR    0
 
 
+/*
+ * Qt includes this typedef and its meta-object system explicitly converts
+ * types to uint.  If these correspond to signal arguments then that conversion
+ * is exposed.  Therefore SIP generates code that uses it.  This definition is
+ * for the cases that SIP is generating non-Qt related bindings with compilers
+ * that don't include it themselves (i.e. MSVC).
+ */
+typedef unsigned int uint;
+
+
 /* Some compatibility stuff to help with handwritten code for SIP v3. */
 #if !defined(ANY)
 #define ANY     void
@@ -195,6 +221,14 @@ extern "C" {
 #define SIPBytes_AS_STRING  PyBytes_AS_STRING
 #define SIPBytes_GET_SIZE   PyBytes_GET_SIZE
 
+#if PY_MINOR_VERSION >= 1
+#define SIP_USE_PYCAPSULE
+#endif
+
+#if PY_MINOR_VERSION < 2
+#define SIP_SUPPORT_PYCOBJECT
+#endif
+
 #else
 
 #define SIPLong_FromLong    PyInt_FromLong
@@ -206,6 +240,12 @@ extern "C" {
 #define SIPBytes_AS_STRING  PyString_AS_STRING
 #define SIPBytes_GET_SIZE   PyString_GET_SIZE
 
+#if PY_MINOR_VERSION >= 7
+#define SIP_USE_PYCAPSULE
+#endif
+
+#define SIP_SUPPORT_PYCOBJECT
+
 #endif
 
 #if !defined(Py_REFCNT)
@@ -248,6 +288,48 @@ typedef int sip_gilstate_t;
 #endif
 
 
+/*
+ * Some convenient function pointers.
+ */
+
+/*
+ * The operation an access function is being asked to perform.
+ */
+typedef enum
+{
+    UnguardedPointer,   /* Return the unguarded pointer. */
+    GuardedPointer,     /* Return the guarded pointer, ie. 0 if it has gone. */
+    ReleaseGuard        /* Release the guard, if any. */
+} AccessFuncOp;
+
+typedef void *(*sipInitFunc)(struct _sipSimpleWrapper *, PyObject *,
+        PyObject *, PyObject **, PyObject **, PyObject **);
+typedef void *(*sipAccessFunc)(struct _sipSimpleWrapper *, AccessFuncOp);
+typedef int (*sipTraverseFunc)(void *, visitproc, void *);
+typedef int (*sipClearFunc)(void *);
+#if PY_MAJOR_VERSION >= 3
+typedef int (*sipGetBufferFunc)(PyObject *, void *, Py_buffer *, int);
+typedef void (*sipReleaseBufferFunc)(PyObject *, void *, Py_buffer *);
+#else
+typedef SIP_SSIZE_T (*sipBufferFunc)(PyObject *, void *, SIP_SSIZE_T, void **);
+typedef SIP_SSIZE_T (*sipSegCountFunc)(PyObject *, void *, SIP_SSIZE_T *);
+#endif
+typedef void (*sipDeallocFunc)(struct _sipSimpleWrapper *);
+typedef void *(*sipCastFunc)(void *, const struct _sipTypeDef *);
+typedef const struct _sipTypeDef *(*sipSubClassConvertFunc)(void **);
+typedef int (*sipConvertToFunc)(PyObject *, void **, int *, PyObject *);
+typedef PyObject *(*sipConvertFromFunc)(void *, PyObject *);
+typedef int (*sipVirtHandlerFunc)(void *, PyObject *, ...);
+typedef void (*sipAssignFunc)(void *, SIP_SSIZE_T, const void *);
+typedef void *(*sipArrayFunc)(SIP_SSIZE_T);
+typedef void *(*sipCopyFunc)(const void *, SIP_SSIZE_T);
+typedef void (*sipReleaseFunc)(void *, int);
+typedef PyObject *(*sipPickleFunc)(void *);
+typedef int (*sipAttrGetterFunc)(const struct _sipTypeDef *, PyObject *);
+typedef PyObject *(*sipVariableGetterFunc)(void *, PyObject *);
+typedef int (*sipVariableSetterFunc)(void *, PyObject *, PyObject *);
+
+
 /*
  * The meta-type of a wrapper type.
  */
@@ -275,13 +357,14 @@ typedef struct _sipWrapperType {
 typedef struct _sipSimpleWrapper {
     PyObject_HEAD
 
-    union {
-        /* C/C++ object pointer. */
-        void *cppPtr;
+    /*
+     * The data, initially a pointer to the C/C++ object, as interpreted by the
+     * access function.
+     */
+    void *data;
 
-        /* Access function. */
-        void *(*afPtr)();
-    } u;
+    /* The optional access function. */
+    sipAccessFunc access_func;
 
     /* Object flags. */
     int flags;
@@ -337,36 +420,6 @@ typedef struct _sipEnumTypeObject {
 } sipEnumTypeObject;
 
 
-/*
- * Some convenient function pointers.
- */
-typedef void *(*sipInitFunc)(sipSimpleWrapper *, PyObject *, PyObject **,
-        int *);
-typedef int (*sipTraverseFunc)(void *, visitproc, void *);
-typedef int (*sipClearFunc)(void *);
-#if PY_MAJOR_VERSION >= 3
-typedef int (*sipGetBufferFunc)(PyObject *, void *, Py_buffer *, int);
-typedef void (*sipReleaseBufferFunc)(PyObject *, void *, Py_buffer *);
-#else
-typedef SIP_SSIZE_T (*sipBufferFunc)(PyObject *, void *, SIP_SSIZE_T, void **);
-typedef SIP_SSIZE_T (*sipSegCountFunc)(PyObject *, void *, SIP_SSIZE_T *);
-#endif
-typedef void (*sipDeallocFunc)(sipSimpleWrapper *);
-typedef void *(*sipCastFunc)(void *, const struct _sipTypeDef *);
-typedef const struct _sipTypeDef *(*sipSubClassConvertFunc)(void **);
-typedef int (*sipConvertToFunc)(PyObject *, void **, int *, PyObject *);
-typedef PyObject *(*sipConvertFromFunc)(void *, PyObject *);
-typedef int (*sipVirtHandlerFunc)(void *, PyObject *, ...);
-typedef void (*sipAssignFunc)(void *, SIP_SSIZE_T, const void *);
-typedef void *(*sipArrayFunc)(SIP_SSIZE_T);
-typedef void *(*sipCopyFunc)(const void *, SIP_SSIZE_T);
-typedef void (*sipReleaseFunc)(void *, int);
-typedef PyObject *(*sipPickleFunc)(void *);
-typedef int (*sipAttrGetterFunc)(const struct _sipTypeDef *, PyObject *);
-typedef PyObject *(*sipVariableGetterFunc)(void *, PyObject *);
-typedef int (*sipVariableSetterFunc)(void *, PyObject *, PyObject *);
-
-
 /*
  * The information describing an encoded type ID.
  */
@@ -466,6 +519,16 @@ typedef struct _sipSubClassConvertorDef {
 } sipSubClassConvertorDef;
 
 
+/*
+ * The different error states of handwritten code.
+ */
+typedef enum {
+    sipErrorNone,       /* There is no error. */
+    sipErrorFail,       /* The error is a failure. */
+    sipErrorContinue    /* It may not apply if a later operation succeeds. */
+} sipErrorState;
+
+
 /*
  * The different Python slot types.
  */
@@ -672,6 +735,9 @@ typedef struct _sipClassTypeDef {
     /* The container information. */
     sipContainerDef ctd_container;
 
+    /* The docstring. */
+    const char *ctd_docstring;
+
     /*
      * The meta-type name, -1 to use the meta-type of the first super-type
      * (normally sipWrapperType).
@@ -752,11 +818,7 @@ typedef struct _sipMappedTypeDef {
     /* The base type information. */
     sipTypeDef mtd_base;
 
-    /*
-     * The container information.  (At the moment only the enum information is
-     * used but mapped types should be able to contain any static methods,
-     * variables, instances etc.)
-     */
+    /* The container information. */
     sipContainerDef mtd_container;
 
     /* The optional assignment function. */
@@ -849,6 +911,9 @@ typedef struct _sipVersionedFunctionDef {
     /* The METH_* flags. */
     int vf_flags;
 
+    /* The docstring. */
+    const char *vf_docstring;
+
     /* The API version range index. */
     int vf_api_range;
 } sipVersionedFunctionDef;
@@ -1255,6 +1320,7 @@ typedef struct _sipAPIDef {
     int (*api_register_attribute_getter)(const sipTypeDef *td,
             sipAttrGetterFunc getter);
     int (*api_is_api_enabled)(const char *name, int from, int to);
+    sipErrorState (*api_bad_callable_arg)(int arg_nr, PyObject *arg);
 
     /*
      * The following are deprecated parts of the public API.
@@ -1285,14 +1351,15 @@ typedef struct _sipAPIDef {
      * The following are not part of the public API.
      */
     int (*api_init_module)(sipExportedModuleDef *client, PyObject *mod_dict);
-    int (*api_parse_args)(int *argsParsedp, PyObject *sipArgs,
+    int (*api_parse_args)(PyObject **parseErrp, PyObject *sipArgs,
             const char *fmt, ...);
-    int (*api_parse_pair)(int *argsParsedp, PyObject *arg0, PyObject *arg1,
+    int (*api_parse_pair)(PyObject **parseErrp, PyObject *arg0, PyObject *arg1,
             const char *fmt, ...);
     void (*api_common_dtor)(sipSimpleWrapper *sipSelf);
-    void (*api_no_function)(int argsParsed, const char *func);
-    void (*api_no_method)(int argsParsed, const char *classname,
-            const char *method);
+    void (*api_no_function)(PyObject *parseErr, const char *func,
+            const char *doc);
+    void (*api_no_method)(PyObject *parseErr, const char *scope,
+            const char *method, const char *doc);
     void (*api_abstract_method)(const char *classname, const char *method);
     void (*api_bad_class)(const char *classname);
     void *(*api_get_cpp_ptr)(sipSimpleWrapper *w, const sipTypeDef *td);
@@ -1328,6 +1395,11 @@ typedef struct _sipAPIDef {
 #endif
     int (*api_deprecated)(const char *classname, const char *method);
     void (*api_keep_reference)(PyObject *self, int key, PyObject *obj);
+    int (*api_parse_kwd_args)(PyObject **parseErrp, PyObject *sipArgs,
+            PyObject *sipKwdArgs, const char **kwdlist, PyObject **unused,
+            const char *fmt, ...);
+    void (*api_add_exception)(sipErrorState es, PyObject **parseErrp);
+    void *(*api_get_address)(struct _sipSimpleWrapper *w);
 } sipAPIDef;
 
 
@@ -1412,6 +1484,7 @@ typedef struct _sipQtAPI {
 #define SIP_TYPE_SCC        0x0010  /* If the type is subject to sub-class convertors. */
 #define SIP_TYPE_ALLOW_NONE 0x0020  /* If the type can handle None. */
 #define SIP_TYPE_STUB       0x0040  /* If the type is a stub. */
+#define SIP_TYPE_NONLAZY    0x0080  /* If the type has a non-lazy method. */
 
 
 /*
@@ -1445,6 +1518,7 @@ typedef struct _sipQtAPI {
 #define sipTypeAllowNone(td)    ((td)->td_flags & SIP_TYPE_ALLOW_NONE)
 #define sipTypeIsStub(td)   ((td)->td_flags & SIP_TYPE_STUB)
 #define sipTypeSetStub(td)  ((td)->td_flags |= SIP_TYPE_STUB)
+#define sipTypeHasNonlazyMethod(td) ((td)->td_flags & SIP_TYPE_NONLAZY)
 
 /*
  * Get various names from the string pool for various data types.
@@ -1502,6 +1576,9 @@ typedef struct _pyqt4QtSignal {
     /* The C++ name and signature of the signal. */
     const char *signature;
 
+    /* The optional docstring. */
+    const char *docstring;
+
     /*
      * If the signal is an overload of regular methods then this points to the
      * code that implements those methods.
