@@ -449,7 +449,7 @@ class Makefile:
 
                 libs.append(self.platform_lib(py_lib))
 
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             if win_exceptions:
                 cflags_exceptions = "CFLAGS_EXCEPTIONS_ON"
                 cxxflags_exceptions = "CXXFLAGS_EXCEPTIONS_ON"
@@ -503,7 +503,7 @@ class Makefile:
             cxxflags_debug = "CXXFLAGS_RELEASE"
             lflags_debug = "LFLAGS_RELEASE"
 
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             if self._threaded:
                 cflags.extend(self.optional_list(cflags_mt))
                 cxxflags.extend(self.optional_list(cxxflags_mt))
@@ -543,6 +543,8 @@ class Makefile:
                     # Note that qmake doesn't define anything for QtHelp.
                     if mod == "QtCore":
                         defines.append("QT_CORE_LIB")
+                    elif mod == "QtDeclarative":
+                        defines.append("QT_DECLARATIVE_LIB")
                     elif mod == "QtGui":
                         defines.append("QT_GUI_LIB")
                     elif mod == "QtMultimedia":
@@ -582,12 +584,14 @@ class Makefile:
                     "QtCore":       "LIBS_CORE",
                     "QtGui":        "LIBS_GUI",
                     "QtNetwork":    "LIBS_NETWORK",
-                    "QtOpenGL":     "LIBS_OPENGL"
+                    "QtOpenGL":     "LIBS_OPENGL",
+                    "QtWebKit":     "LIBS_WEBKIT"
                 }
 
                 # For Windows: the dependencies between Qt libraries.
                 qdepmap = {
                     "QtAssistant":      ("QtNetwork", "QtGui", "QtCore"),
+                    "QtDeclarative":    ("QtNetwork", "QtGui", "QtCore"),
                     "QtGui":            ("QtCore", ),
                     "QtHelp":           ("QtSql", "QtGui", "QtCore"),
                     "QtMultimedia":     ("QtGui", "QtCore"),
@@ -655,7 +659,7 @@ class Makefile:
                 # Windows needs the version number appended if Qt is a DLL.
                 qt_lib = self.config.qt_lib
 
-                if self.generator in ("MSVC", "MSVC.NET", "BMAKE") and win_shared:
+                if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE") and win_shared:
                     qt_lib = qt_lib + version_to_string(self.config.qt_version).replace(".", "")
 
                     if self.config.qt_edition == "non-commercial":
@@ -702,7 +706,7 @@ class Makefile:
             libs.extend(self.optional_list("LIBS_OPENGL"))
 
         if self._qt or self._opengl:
-            if self.config.qt_version < 0x040000 or "QtGui" in self._qt:
+            if self.config.qt_version < 0x040000 or self._opengl or "QtGui" in self._qt:
                 incdir.extend(self.optional_list("INCDIR_X11"))
                 libdir.extend(self.optional_list("LIBDIR_X11"))
                 libs.extend(self.optional_list("LIBS_X11"))
@@ -766,10 +770,11 @@ class Makefile:
                 lib = lib + "_debug"
 
         if sys.platform == "win32" and "shared" in self.config.qt_winconfig.split():
-            if (mname in ("QtCore", "QtDesigner", "QtGui", "QtHelp",
-                          "QtMultimedia", "QtNetwork", "QtOpenGL", "QtScript",
-                          "QtScriptTools", "QtSql", "QtSvg", "QtTest",
-                          "QtWebKit", "QtXml", "QtXmlPatterns", "phonon") or
+            if (mname in ("QtCore", "QtDeclarative", "QtDesigner", "QtGui",
+                          "QtHelp", "QtMultimedia", "QtNetwork", "QtOpenGL",
+                          "QtScript", "QtScriptTools", "QtSql", "QtSvg",
+                          "QtTest", "QtWebKit", "QtXml", "QtXmlPatterns",
+                          "phonon") or
                 (self.config.qt_version >= 0x040200 and mname == "QtAssistant")):
                 lib = lib + "4"
 
@@ -815,7 +820,11 @@ class Makefile:
         flags = []
         prefix = self.optional_string("RPATH")
 
-        if prefix:
+        if prefix == "":
+            # This was renamed in Qt v4.7.
+            prefix = self.optional_string("LFLAGS_RPATH")
+
+        if prefix != "":
             for r in rpaths:
                 flags.append(_quote(prefix + r))
 
@@ -827,7 +836,7 @@ class Makefile:
         clib is the library name in cannonical form.
         framework is set of the library is implemented as a MacOS framework.
         """
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             plib = clib + ".lib"
         elif sys.platform == "darwin" and framework:
             plib = "-framework " + clib
@@ -845,7 +854,7 @@ class Makefile:
         """
         prl_libs = []
 
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             prl_name = os.path.join(self.config.qt_lib_dir, clib + ".prl")
         elif sys.platform == "darwin" and framework:
             prl_name = os.path.join(self.config.qt_lib_dir, clib + ".framework", clib + ".prl")
@@ -933,7 +942,7 @@ class Makefile:
                 bdict[i] = ""
 
         # Generate the list of objects.
-        if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             ext = ".obj"
         else:
             ext = ".o"
@@ -1031,7 +1040,7 @@ class Makefile:
 
         libs = []
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             libdir_prefix = "/LIBPATH:"
         else:
             libdir_prefix = "-L"
@@ -1068,7 +1077,7 @@ class Makefile:
         else:
             mfile.write(".SUFFIXES: .c .cpp .cc .cxx .C\n\n")
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             mfile.write("""
 {.}.cpp{}.obj::
 \t$(CXX) -c $(CXXFLAGS) $(CPPFLAGS) -Fo @<<
@@ -1510,7 +1519,7 @@ class ModuleMakefile(Makefile):
         mfile.write("\n")
 
         if self.static:
-            if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
+            if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
                 mfile.write("LIB = %s\n" % self.required_string("LIB"))
             elif self.generator == "MINGW":
                 mfile.write("AR = %s\n" % self.required_string("LIB"))
@@ -1544,7 +1553,7 @@ class ModuleMakefile(Makefile):
 
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             if self.static:
                 mfile.write("\t$(LIB) /OUT:$(TARGET) @<<\n")
                 mfile.write("\t  $(OFILES)\n")
@@ -1726,7 +1735,7 @@ class ProgramMakefile(Makefile):
         if self.generator != "BMAKE":
             build.append(source)
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             build.append("-Fe")
             build.append("/link")
             libdir_prefix = "/LIBPATH:"
@@ -1779,7 +1788,7 @@ class ProgramMakefile(Makefile):
     def finalise(self):
         """Finalise the macros for a program Makefile.
         """
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             self.LFLAGS.append("/INCREMENTAL:NO")
 
         if self._manifest:
@@ -1836,7 +1845,7 @@ class ProgramMakefile(Makefile):
 
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
-        if self.generator in ("MSVC", "MSVC.NET"):
+        if self.generator in ("MSVC", "MSVC.NET", "MSBUILD"):
             mfile.write("\t$(LINK) $(LFLAGS) /OUT:$(TARGET) @<<\n")
             mfile.write("\t  $(OFILES) $(LIBS)\n")
             mfile.write("<<\n")
@@ -2299,8 +2308,10 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
         if line and line[0] != "#":
             assstart = line.find("+")
             if assstart > 0 and line[assstart + 1] == '=':
+                adding = True
                 assend = assstart + 1
             else:
+                adding = False
                 assstart = line.find("=")
                 assend = assstart
 
@@ -2311,6 +2322,11 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                 # Remove the escapes for any quotes.
                 rhs = rhs.replace(r'\"', '"').replace(r"\'", "'")
 
+                if adding and rhs != "":
+                    orig_rhs = raw.get(lhs)
+                    if orig_rhs is not None:
+                        rhs = orig_rhs + " " + rhs
+
                 raw[lhs] = rhs
 
         line = f.readline()
